//MyEntity.cpp

#include "stdafx.h"
#include "MyProperty.h"
#include "MyEntity.h"
#include "MyComponent.h"
#include "MyEntityManager.h"


MyEntity::MyEntity( MyEntityManager* pManager, entity_uid uid ) : m_pManager( pManager), m_uid( uid )
{
}
//--------------------------------------------------------------------------------------
MyEntity::~MyEntity()
{
	{
		ComponetTable::iterator it = m_components.begin();
		ComponetTable::iterator end = m_components.end();
		for( ; it != end; ++it )
		{
			delete it->second;
		}
		m_components.clear();
	}
}
//--------------------------------------------------------------------------------------
void MyEntity::Remove()
{
	m_pManager->AddEntityToDeleteList( this );
}
//--------------------------------------------------------------------------------------
void MyEntity::Update( int timeDiff )
{
	ComponetTable::iterator it = m_components.begin();
	ComponetTable::iterator end = m_components.end();
	while( it != end )
	{
		it->second->Update( timeDiff );
		++it;
	}
}
//--------------------------------------------------------------------------------------
void MyEntity::AddComponent( const component_uid& uid )
{
	MyComponent* component = m_pManager->AddComponentToEntity( uid, this );
}
//--------------------------------------------------------------------------------------
MyComponent* MyEntity::GetComponent( const component_uid& uid )
{
	ComponetTable::iterator it = m_components.find( uid );
	if( it != m_components.end() )
	{
		return it->second;
	}

	return NULL;
}
//--------------------------------------------------------------------------------------
void MyEntity::DirectDeleteComponent( const component_uid& uid )
{
	ComponetTable::iterator it = m_components.find( uid );
	if( it != m_components.end() )
	{
		delete it->second;
		m_components.erase( it );
	}
}
//--------------------------------------------------------------------------------------
void MyEntity::DirectSetComponent( const component_uid& uid, MyComponent* pComponent )
{
	m_components.insert( std::make_pair( uid, pComponent ) );
}
//--------------------------------------------------------------------------------------


