//MyEntityManager.cpp

#include "stdafx.h"
#include "MyEntityManager.h"
#include "MyProperty.h"
#include "MyEntity.h"
#include "MyComponentFactory.h"
#include "MyComponent.h"


MyEntityManager::MyEntityManager( MyStageManager* manager )
{
	m_pStageManager = manager;
}
//---------------------------------------------------------------------------------------------
MyEntityManager::~MyEntityManager()
{
	std::list< MyEntity* >::iterator it = m_entityList.begin();
	while( it != m_entityList.end() )
	{
		MyEntity* e = *it;
		e->Dec();
		++it;
	}
	m_entityList.clear();
}
//---------------------------------------------------------------------------------------------
MyStageManager* MyEntityManager::GetStageManager()
{
	return m_pStageManager;
}
//---------------------------------------------------------------------------------------------
MyEntity* MyEntityManager::CreateEntity( entity_uid uid )
{
	MyEntity* entity = new MyEntity( this, uid );
	m_entityList.push_back( entity );
	return entity;
}
//---------------------------------------------------------------------------------------------
void MyEntityManager::AddEntityToDeleteList( MyEntity* pEntity )
{
	std::list< MyEntity* >::iterator it = m_entityToDelete.begin();
	while( it != m_entityToDelete.end() )
	{
		if( *it == pEntity )
			return;
		++it;
	}
}
//---------------------------------------------------------------------------------------------
void MyEntityManager::Update( int timeDiff )
{
	std::list< MyEntity* >::iterator it;

	it = m_entityToDelete.begin();
	while( it != m_entityToDelete.end() )
	{
		std::list< MyEntity* >::iterator at = m_entityList.begin();
		while( at != m_entityList.end() )
		{
			if( *it == *at )
			{
				( *at )->Dec();
				m_entityList.erase( at );
				break;
			}
			++at;
		}
		++it;
	}
	m_entityToDelete.clear();

	//Update entity list
	it = m_entityList.begin();
	while( it != m_entityList.end() )
	{
		( *it )->Update( timeDiff );
		++it;
	}
}
//---------------------------------------------------------------------------------------------
void MyEntityManager::RegisterComponent( MyComponentFactory* factory )
{
	std::map< component_uid, MyComponentFactory* >::iterator it = m_componentFactory.find( factory->GetUID() );
	if( it != m_componentFactory.end() )
	{
		MyComponentFactory* f = it->second;
		delete f;
	}
	m_componentFactory.insert( std::make_pair( factory->GetUID(), factory ) );
}
//---------------------------------------------------------------------------------------------
MyComponent* MyEntityManager::AddComponentToEntity( component_uid uid, MyEntity* entity )
{
	std::map< component_uid, MyComponentFactory* >::iterator it = m_componentFactory.find( uid );
	if( it != m_componentFactory.end() )
	{
		MyComponentFactory* factory = it->second;
		MyComponent* component = factory->CreateComponent( entity, this );
		entity->DirectDeleteComponent( uid );
		entity->DirectSetComponent( uid, component );
		return component;
	}

	return NULL;
}