//MyStageManager.cpp

#include "stdafx.h"
#include "MyStageManager.h"
#include "MyStage.h"
#include "MyStageFactory.h"

#define TEMP_TIMER   1

#ifdef TEMP_TIMER
LARGE_INTEGER g_freq;
LARGE_INTEGER g_end;

float GetSecond()
{
	LARGE_INTEGER curr;
	QueryPerformanceCounter( &curr );
	LONGLONG elapsed = curr.QuadPart - g_end.QuadPart;
	return ( (float)elapsed/g_freq.QuadPart );
}
#endif


MyStageManager::MyStageManager()
{
	m_mainStage = NULL;
	m_lastTime = 0;

#ifdef TEMP_TIMER
	QueryPerformanceFrequency( &g_freq );
	QueryPerformanceCounter( &g_end );
#endif
}
//--------------------------------------------------------------------------------------
MyStageManager::~MyStageManager()
{
	{
		std::list< MyStage* >::iterator it = m_stageColl.begin();
		for( ; it != m_stageColl.end(); ++it )
		{
			MyStage* s = *it;
			delete s;
		}
		m_stageColl.clear();
	}
	{
		std::vector< MyStageFactory* >::iterator it	= m_factories.begin();
		for( ; it != m_factories.end(); ++it )
		{
			MyStageFactory* factory = *it;
			delete factory;
		}
		m_factories.clear();
	}
	m_stageCollToRemove.clear();
}
//--------------------------------------------------------------------------------------
MyStage* MyStageManager::CreateStage( const std::string& name )
{
	int iSize = (int)m_factories.size();
	for( int i = 0; i < iSize; ++i )
	{
		MyStage* stage = m_factories[i]->CreateStage( this );
		return stage;
	}
	return NULL;
}
//--------------------------------------------------------------------------------------
void MyStageManager::RegisterStage( MyStageFactory* factory )
{
	int iSize = (int)m_factories.size();
	for( int i = 0; i < iSize; ++i )
	{
		if( factory->m_name == m_factories[i]->m_name )
		{
			delete factory;
			return;
		}
	}
	m_factories.push_back( factory );
}
//--------------------------------------------------------------------------------------
bool MyStageManager::Update()
{
	//Update World
	if( m_mainStage )
	{
		if( m_stageColl.empty() )
		{
			//printf( "No m_stageColl available\n" );
			return false;
		}
		//printf( "Use first stage as MainStage\n" );
		std::list< MyStage* >::iterator it = m_stageColl.begin();
		m_mainStage = *it;
		m_mainStage->BecomeMainStage();
	}

	if( m_lastTime == 0 )
	{
		m_lastTime = GetSecond();
	}
	
	int timeDiff = int( GetSecond() - m_lastTime ) * 1000;
	m_lastTime = GetSecond();

	if( m_mainStage )
		m_mainStage->Update( timeDiff );

	if( m_stageCollToRemove.empty() == false )
	{
		//printf( "remove m_stageCollToRemove..." );
		std::list< MyStage* >::iterator it = m_stageCollToRemove.begin();
		while( it != m_stageCollToRemove.end() )
		{
			MyStage* stage = *it;
			if( stage == m_mainStage )
			{
				m_mainStage->BecomeMainStage();
				m_mainStage = NULL;
			}
			delete stage;
			++it;
		}
		m_stageCollToRemove.clear();
		//printf( "done\n" );
	}
	return true;
}
//--------------------------------------------------------------------------------------
void MyStageManager::AddStage( MyStage* stage )
{
	//printf( "Add stage..." );
	std::list< MyStage* >::iterator it = m_stageColl.begin();
	while( it != m_stageColl.end() )
	{
		if( *it == stage )
			return;
		++it;
	}
	m_stageColl.push_back( stage );
	//printf( "done\n" );
}
//--------------------------------------------------------------------------------------
void MyStageManager::RemoveStage( MyStage* stage )
{
	std::list< MyStage* >::iterator it = m_stageColl.begin();
	while( it != m_stageColl.end() )
	{
		if( *it == stage )
		{
			if( m_mainStage == *it )
			{
				m_mainStage->BecomeMainStage();
				m_mainStage = NULL;
			}
			m_stageColl.erase( it );
			return;
		}
		++it;
	}
}
//--------------------------------------------------------------------------------------
void MyStageManager::StageToRemove( MyStage* stage )
{
	std::list< MyStage* >::iterator it = m_stageCollToRemove.begin();
	while( it != m_stageCollToRemove.end() )
	{
		if( *it == stage )
			return;
		++it;
	}
	m_stageCollToRemove.push_back( stage );
}
//--------------------------------------------------------------------------------------
